<?php

declare(strict_types=1);

namespace Drupal\firebase_php_integration\Plugin\Action;

use Drupal\Core\Action\ActionBase;
use Drupal\Core\Action\Attribute\Action;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\firebase_php\FirebasePhpMessagingApiInterface;
use Drupal\firebase_php\Exception\FirebasePhpInvalidArgumentException;
use Drupal\firebase_php\Exception\FirebasePhpInvalidTokenException;
use Drupal\push_notifications_registration_tokens\RegistrationTokenInterface;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Validates Firebase registration tokens and deletes invalid ones.
 */
#[Action(
  id: 'registration_token_validate_action',
  label: new TranslatableMarkup('Validate registration token'),
  category: new TranslatableMarkup('Push Notifications'),
  type: 'pn_registration_token',
)]
class RegistrationTokenValidateAction extends ActionBase implements ContainerFactoryPluginInterface {

  public function __construct(
    array $configuration,
    string $plugin_id,
    mixed $plugin_definition,
    protected ?FirebasePhpMessagingApiInterface $firebaseMessagingApi,
    protected LoggerInterface $logger,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): static {
    // In tests, Firebase won't be configured.
    $firebaseMessagingApi = NULL;
    try {
      $firebaseMessagingApi = $container->get('firebase_php.messaging_drupal_api');
    }
    // @phpstan-ignore-next-line We do not need to rethrow this exception.
    catch (\Exception $e) {
      // Firebase service is not available. Log and continue.
      $logger = $container->get('logger.factory')->get('push_notifications_registration_tokens');
      $logger->warning('Firebase PHP messaging service is not available: @message', [
        '@message' => $e->getMessage(),
      ]);
    }

    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $firebaseMessagingApi,
      $container->get('logger.factory')->get('push_notifications_registration_tokens'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function execute(?EntityInterface $entity = NULL): void {
    if (is_null($entity)) {
      $this->logger->error('No entity passed to ValidateRegistrationTokenAction.');
      return;
    }

    if (!($entity instanceof RegistrationTokenInterface)) {
      $this->logger->error('Invalid entity passed to ValidateRegistrationTokenAction.');
      return;
    }

    // Check if Firebase service is available.
    if (is_null($this->firebaseMessagingApi)) {
      $this->logger->error('Firebase messaging service is not available. Please configure the Firebase PHP module.');
      $this->messenger()->addError($this->t('Firebase is not configured. Cannot validate registration token @id.', [
        '@id' => $entity->id(),
      ]));
      return;
    }

    $token_field = $entity->get('token');
    if ($token_field->isEmpty()) {
      $this->messenger()->addError($this->t('Registration token @id has no token value.', [
        '@id' => $entity->id(),
      ]));
      return;
    }

    $token = $token_field->value;

    try {
      $is_valid = $this->firebaseMessagingApi->validateToken($token);

      if ($is_valid) {
        // The token is valid, so update the changed timestamp.
        $entity->set('changed', time());
        $entity->save();

        $this->messenger()->addStatus($this->t('Token @id is valid and has been updated.', [
          '@id' => $entity->id(),
        ]));

        $this->logger->info('Token @id validated successfully.', [
          '@id' => $entity->id(),
        ]);
      }
      else {
        // The token is invalid, so delete it.
        $entity_id = $entity->id();
        $entity->delete();

        $this->messenger()->addWarning($this->t('Token @id was invalid and has been deleted.', [
          '@id' => $entity_id,
        ]));

        $this->logger->info('Invalid token @id has been deleted.', [
          '@id' => $entity_id,
        ]);
      }
    }
    catch (FirebasePhpInvalidArgumentException $e) {
      $this->logger->error('Invalid argument when validating token @id: @message', [
        '@id' => $entity->id(),
        '@message' => $e->getMessage(),
      ]);
      $this->messenger()->addError($this->t('Invalid token format for token @id.', [
        '@id' => $entity->id(),
      ]));
    }
    catch (FirebasePhpInvalidTokenException $e) {
      // The token is invalid, so delete it.
      $entity_id = $entity->id();
      $entity->delete();

      $this->logger->info('Invalid token @id detected and deleted: @message', [
        '@id' => $entity_id,
        '@message' => $e->getMessage(),
      ]);
      $this->messenger()->addWarning($this->t('Token @id was invalid and has been deleted.', [
        '@id' => $entity_id,
      ]));
    }
    // @phpstan-ignore-next-line We do not need to rethrow this exception.
    catch (\Exception $e) {
      $this->logger->error('Error validating token @id: @message', [
        '@id' => $entity->id(),
        '@message' => $e->getMessage(),
      ]);
      $this->messenger()->addError($this->t('Error validating token @id. Check logs for details.', [
        '@id' => $entity->id(),
      ]));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function access($object, ?AccountInterface $account = NULL, $return_as_object = FALSE): bool {
    $access = $object->access('update', $account, TRUE)
      ->andIf($object->access('delete', $account, TRUE))
      ->andIf($object->access('view', $account, TRUE));

    return $return_as_object ? $access : $access->isAllowed();
  }

}
